import time
import urllib.request
import re
import lxml
from pyquery import PyQuery as pq

def checkExist(patternList,str):
    for p in patternList:
        if(re.match(p, str)):
            return True
    return False

def wirteSkipList(str):
    with open('./skip','a') as f:
        f.write(f'{str}\n')

def getResopnseBody(url):
    response = urllib.request.urlopen(url)
    return response.read()

def fmtNewLine(str, subitem = 0):
    return '    '*subitem + f'{str}\n'

def fmtIndexItemLine(str, subitem = 0):
    return '    '*subitem + f'1. {str}\n\n'

def fmtItemLine(str, subitem = 0):
    return '    '*subitem + f'- {str}\n\n'

def fmtHeadLine(str,size = 3):
    return '#'*size + f' {str}\n'

def fmtLink(str,url):
    return f'[{str}]({url})'

def dumpToFile(list, sikpWords=[], filePath = './res.md'):
    with open(filePath,'w') as optput:
        tabCount = 0
        for name,info in list.items():
            optput.write(fmtHeadLine(fmtLink(name, info.teacherLink),3))
            # dump basic Info
            optput.write(fmtItemLine('基本信息', tabCount))
            tabCount+=1
            buf = re.split('\n',info.basicInfo)
            for i in buf:
                optput.write(fmtItemLine(i, tabCount))
            tabCount -= 1
            # dump another info
            for i in info.infoList:
                if(checkExist(sikpWords,i.title)):
                    continue
                optput.write(fmtItemLine(i.title, tabCount))
                tabCount += 1
                for index in range(len(i.itemContent)):
                    if i.itemContent[index] == '' and i.itemTitle[index] == '':
                        continue
                    if (checkExist(sikpWords, i.itemTitle[index])):
                        continue
                    optput.write(fmtItemLine(i.itemTitle[index],tabCount))
                    if(re.match(r'（\W*[1-9]\d*\W*）',i.itemContent[index])):
                        content = re.split(r'（\W*[1-9]\d*\W*）',i.itemContent[index].strip())
                        for item in content:
                            if(item == ''):
                                continue
                            optput.write(fmtIndexItemLine(item, tabCount))
                    else:
                        optput.write(fmtNewLine(i.itemContent[index],tabCount))
                tabCount -= 1
            tabCount -= 1

class InfoItem():
    def __init__(self, itemCtx):
        self.itemTitle = []
        self.itemContent = []
        self.title = itemCtx('h3').text()
        boxList = itemCtx.find('.mi-box')
        for box in boxList.items():
            subTitle = box('.mib-t')
            if (subTitle == 0):
                self.itemTitle.append('')
            else:
                self.itemTitle.append(subTitle.text())
            self.itemContent.append(box('.mib-c').text().replace(u'\xa0', ' ').replace('&nbsp', ' '))

        # print(self.itemTitle)
        # print(self.itemContent)


class TeacherInfo():
    def __init__(self, name, linkOfInfo, ctx):
        self.teacherNeme = name
        self.teacherLink = linkOfInfo
        self.basicInfo = ''
        self.infoList = []

        pageVerision = ctx('body').attr('id')
        if pageVerision == 'myzone3':
            self.paseMyzone(ctx('body'))
        elif pageVerision == 'myzone2':
            self.paseMyzone(ctx('body'))
        else:
            Exception('No page handler')

        print('done')

    def paseMyzone(self, ctx):
        content = ctx('#main-content .container')
        self.basicInfo = content('.b-pinfo .bp-enty').text()
        itemList = content('.m-itme')
        for item in itemList.items():
            self.infoList.append(InfoItem(item))


# Press the green button in the gutter to run the script.
if __name__ == '__main__':
    url = 'https://scce.ucas.ac.cn/index.php/zh-CN/szdw/graduateteacher'
    print(f'get HTML... form{url}')
    context = getResopnseBody(url)
    dom = pq(context)
    teacherList = {}
    try:
        for item in dom('.yp_ity a').items():
            name = item.text()
            link = item.attr('href')
            print(f'Getting Info of {name} from {link} ...')

            try:
                content = getResopnseBody(link)
                ctx = pq(content)
                teacherList[name] = TeacherInfo(name, link, ctx)
            except UnicodeEncodeError:
                wirteSkipList(name)
                print('Invalid link, skip')
            except lxml.etree.XMLSyntaxError:
                wirteSkipList(name)
                print('Blank Page, skip')
            except urllib.error.HTTPError:
                print('get 502 error, try again')
                time.sleep(20)
                content = getResopnseBody(link)
                ctx = pq(content)
                teacherList[name] = TeacherInfo(name, link, ctx)
            except BaseException as e:
                wirteSkipList(name)
                print(f'Unknown exception {e}')
                print('Skip.')
            time.sleep(10)
    finally:
        dumpToFile(teacherList, ['专利','出版'])
